from traxible.traxodule import Traxodule


class Docker(Traxodule):
    def install(self):
        self.run(
            [
                "sudo apt-get install"
                " -y apt-transport-https"
                " ca-certificates"
                " curl gnupg-agent"
                " software-properties-common",
                "curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo apt-key add -",
                "sudo apt-key fingerprint 0EBFCD88",
                'sudo add-apt-repository"'
                '" deb [arch=amd64] https://download.docker.com/linux/ubuntu '
                '$(lsb_release -cs) stable"',
                "sudo apt-get update",
                "sudo apt-get install -y docker-ce docker-ce-cli containerd.io",
                "sudo gpasswd -a ubuntu docker",
            ]
        )

    def list(self):
        self.run(["docker container list"])

    def network(self, name, subnet):
        self.run(
            [
                "docker network create --subnet={subnet} {name} || echo fine".format(
                    subnet=subnet, name=name
                )
            ],
            block=False,
        )

    def create(
        self,
        name,
        image,
        network_name=None,
        port_expose=None,
        environ=None,
        restart="always",
        ip=None,
        command=None,
        force=False,
    ):
        command_line = "docker container run -dit --name " + name + " "

        if network_name is not None:
            command_line += " --network " + network_name

        if port_expose is not None:

            for port in port_expose:
                command_line += " -e " + str(port)

        if environ is not None:
            for key, value in environ.items():
                command_line += f" -e {key}={value} "

        if restart is not None:
            command_line += " --restart " + restart

        if ip is not None:
            command_line += " --ip " + ip

        command_line += " " + image + " "

        if command is not None:
            command_line += " " + command

        if force is True:
            self.run(["docker container rm -f {name}".format(name=name)], block=False)

        self.run([command_line])



from traxible.traxodule import Traxodule


class Deb(Traxodule):
    def update(self):
        self.run(["sudo apt update", "sudo apt upgrade -y"])

    def install(self, packages):
        self.run([f"sudo apt install -y {packages}"])



import ipaddress
from fire import Fire


def ip_in_cidr(cidr, following=None):

    condition = following is None
    for i, ip in enumerate(ipaddress.ip_network(cidr)):
        if i == 0:
            continue

        if condition:
            return str(ip)

        if following == str(ip):
            condition = True

    return None


if __name__ == "__main__":
    Fire()



import tempfile
from traxible.traxodule import Traxodule


class Nginx(Traxodule):
    def install(self):
        self.run(["sudo apt install -y nginx certbot python3-certbot-nginx"])

    def add_docker(self, domain, docker_ip):
        new_conf = f"""
server {{
    index index.html index.htm;
    server_name {domain};

    location / {{
        proxy_pass http://{docker_ip}:4242;
        proxy_redirect off;
        proxy_set_header Host $host;
    }}

    listen [::]:443 ssl ipv6only=on;
    listen 443 ssl;

}}
"""
        with tempfile.NamedTemporaryFile(mode="w") as file:
            file.write(new_conf)
            file.flush()
            self.put(
                file.name,
                "/etc/nginx/sites-enabled/{domain}".format(domain=domain),
                sudo=True,
            )

    def certbot(self, domain, mail):
        self.run(
            [
                f"sudo certbot --nginx  --non-interactive"
                f"--redirect --agree-tos -m {mail} --domains {domain}"
            ]
        )

    def restart(self):
        self.run(["sudo service nginx restart"])



from fire import Fire
from fabric import Connection


class Traxible:
    """
    This is main module that allows basic remote operations
    sush as ssh connection, send command or files, reconnect, ...

    """

    def __init__(self, host):
        self.host = host
        self.conn = Connection(self.host)

    def run(self, commands, block=True):
        for command in commands:
            print("> ", command)
            try:
                result = self.conn.run(command)
                print(result.exited, "=" * 60)

            except Exception as error:
                if block is True:
                    raise error

    def put(self, source, destination, sudo=False):
        if sudo is False:
            return self.conn.put(source, destination)

        self.conn.put(source, "/tmp/efe9e3023a502013767a0559a94a5e")
        return self.run([f"sudo mv /tmp/efe9e3023a502013767a0559a94a5e {destination}"])

    def reconnect(self):
        self.conn.close()
        self.conn.open()


if __name__ == "__main__":
    Fire(Traxible)
